@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2010-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2010-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import datetime
 
-from log import m18n, m18nc, m18ncE, logWarning, logException, logDebug
+from log import logWarning, logException, logDebug
+from mi18n import i18n, i18nc, i18ncE
 from sound import Voice
 from tile import Tile, TileList
 from meld import Meld, MeldList
-from common import Internal, Debug, Options, long
-from common import unicode, unicodeString
+from common import Internal, Debug, Options, StrMixin
 from wind import Wind
 from dialogs import Sorry
 
@@ -35,7 +35,7 @@ from dialogs import Sorry
 # will be called anyway
 
 
-class Message(object):
+class Message:
 
     """those are the message types between client and server. They have no state
     i.e. they never hold real attributes. They only describe the message and actions upon it"""
@@ -45,7 +45,7 @@ class Message(object):
     def __init__(self, name=None, shortcut=None):
         """those are the english values"""
         self.name = name or self.__class__.__name__.replace('Message', '')
-        self.i18nName = self.name
+        self.__i18nName = None
         self.shortcut = shortcut
         # do not use a numerical value because that could easier
         # change with software updates
@@ -53,12 +53,16 @@ class Message(object):
         msgName = self.name.replace(' ', '')
         assert className == msgName, '%s != %s' % (className, msgName)
 
+    @property
+    def i18nName(self):
+        """only translate when needed - most messages never need this"""
+        if self.__i18nName is None:
+            self.__i18nName = i18nc('kajongg', self.name)
+        return self.__i18nName
+
     def __str__(self):
         return self.name
 
-    def __unicode__(self):
-        return unicodeString(self.name)
-
     def __repr__(self):
         return 'Message.{}'.format(self.name)
 
@@ -72,9 +76,9 @@ class Message(object):
         # pylint: disable=too-many-return-statements
         cls = value.__class__
         if cls in (Tile, TileList, Meld, MeldList):
-            return str(value).encode()
+            return str(value)
         elif isinstance(value, Wind):
-            return str(value).encode()
+            return str(value)
         elif isinstance(value, Message):
             return value.name
         elif isinstance(value, (list, tuple)):
@@ -85,7 +89,7 @@ class Message(object):
         elif isinstance(value, dict):
             return dict((Message.jelly('key', x[0]), Message.jelly('value', x[1])) for x in value.items())
         else:
-            if not isinstance(value, (int, long, bytes, unicode, float, type(None))):
+            if not isinstance(value, (int, bytes, str, float, type(None))):
                 raise Exception(
                     'callRemote got illegal arg: %s %s(%s)' %
                     (key, type(value), str(value)))
@@ -123,11 +127,10 @@ class ClientMessage(Message):
 
     def __init__(self, name=None, shortcut=None):
         Message.__init__(self, name, shortcut)
-        self.i18nName = m18nc('kajongg', self.name)
 
     def buttonCaption(self):
         """localized, with a & for the shortcut"""
-        i18nShortcut = m18nc(
+        i18nShortcut = i18nc(
             'kajongg game dialog:Key for ' +
             self.name,
             self.shortcut)
@@ -135,7 +138,7 @@ class ClientMessage(Message):
 
     def toolTip(self, dummyButton, dummyTile):
         """returns text and warning flag for button and text for tile for button and text for tile"""
-        txt = u'toolTip is not defined for %s' % self.name
+        txt = 'toolTip is not defined for %s' % self.name
         logWarning(txt)
         return txt, True, ''
 
@@ -155,6 +158,12 @@ class ClientMessage(Message):
         table.abort(errMsg)
 
 
+class MessageAbort(ClientMessage):
+
+    """If a client aborts, the server will set the answer for all open requests
+    to Message.AbortMessage"""
+
+
 class NotifyAtOnceMessage(ClientMessage):
 
     """those classes are for messages from clients that might have to be relayed to the
@@ -194,7 +203,7 @@ class PungChowMessage(NotifyAtOnceMessage):
         NotifyAtOnceMessage.__init__(self, name=name, shortcut=shortcut)
 
     def toolTip(self, button, dummyTile):
-        """decorate the action button which will send this message"""
+        """for the action button which will send this message"""
         myself = button.client.game.myself
         maySay = myself.sayable[self]
         if not maySay:
@@ -203,7 +212,7 @@ class PungChowMessage(NotifyAtOnceMessage):
         warn = False
         if myself.originalCall and myself.mayWin:
             warn = True
-            txt.append(m18n('saying %1 violates Original Call',
+            txt.append(i18n('saying %1 violates Original Call',
                             self.i18nName))
         dangerousMelds = myself.maybeDangerous(self)
         if dangerousMelds:
@@ -214,16 +223,16 @@ class PungChowMessage(NotifyAtOnceMessage):
                     'only some claimable melds are dangerous: %s' %
                     dangerousMelds)
             if len(dangerousMelds) == 1:
-                txt.append(m18n(
+                txt.append(i18n(
                     'claiming %1 is dangerous because you will have to discard a dangerous tile',
                     lastDiscard.name()))
             else:
                 for meld in dangerousMelds:
-                    txt.append(m18n(
+                    txt.append(i18n(
                         'claiming %1 for %2 is dangerous because you will have to discard a dangerous tile',
                         lastDiscard.name(), str(meld)))
         if not txt:
-            txt = [m18n('You may say %1', self.i18nName)]
+            txt = [i18n('You may say %1', self.i18nName)]
         return '<br><br>'.join(txt), warn, ''
 
 
@@ -233,8 +242,8 @@ class MessagePung(PungChowMessage, ServerMessage):
 
     def __init__(self):
         PungChowMessage.__init__(self,
-                                 name=m18ncE('kajongg', 'Pung'),
-                                 shortcut=m18ncE('kajongg game dialog:Key for Pung', 'P'))
+                                 name=i18ncE('kajongg', 'Pung'),
+                                 shortcut=i18ncE('kajongg game dialog:Key for Pung', 'P'))
 
     def serverAction(self, table, msg):
         """the server mirrors that and tells all others"""
@@ -251,8 +260,8 @@ class MessageKong(NotifyAtOnceMessage, ServerMessage):
 
     def __init__(self):
         NotifyAtOnceMessage.__init__(self,
-                                     name=m18ncE('kajongg', 'Kong'),
-                                     shortcut=m18ncE('kajongg game dialog:Key for Kong', 'K'))
+                                     name=i18ncE('kajongg', 'Kong'),
+                                     shortcut=i18ncE('kajongg game dialog:Key for Kong', 'K'))
 
     def serverAction(self, table, msg):
         """the server mirrors that and tells all others"""
@@ -262,7 +271,7 @@ class MessageKong(NotifyAtOnceMessage, ServerMessage):
             table.declareKong(msg.player, Meld(msg.args[0]))
 
     def toolTip(self, button, dummyTile):
-        """decorate the action button which will send this message"""
+        """for the action button which will send this message"""
         myself = button.client.game.myself
         maySay = myself.sayable[self]
         if not maySay:
@@ -272,10 +281,10 @@ class MessageKong(NotifyAtOnceMessage, ServerMessage):
         if myself.originalCall and myself.mayWin:
             warn = True
             txt.append(
-                m18n('saying Kong for %1 violates Original Call',
+                i18n('saying Kong for %1 violates Original Call',
                      Tile(maySay[0][0]).name()))
         if not txt:
-            txt = [m18n('You may say Kong for %1', Tile(maySay[0][0]).name())]
+            txt = [i18n('You may say Kong for %1', Tile(maySay[0][0]).name())]
         return '<br><br>'.join(txt), warn, ''
 
     def clientAction(self, client, move):
@@ -292,8 +301,8 @@ class MessageChow(PungChowMessage, ServerMessage):
 
     def __init__(self):
         PungChowMessage.__init__(self,
-                                 name=m18ncE('kajongg', 'Chow'),
-                                 shortcut=m18ncE('kajongg game dialog:Key for Chow', 'C'))
+                                 name=i18ncE('kajongg', 'Chow'),
+                                 shortcut=i18ncE('kajongg game dialog:Key for Chow', 'C'))
 
     def serverAction(self, table, msg):
         """the server mirrors that and tells all others"""
@@ -326,8 +335,8 @@ class MessageMahJongg(NotifyAtOnceMessage, ServerMessage):
 
     def __init__(self):
         NotifyAtOnceMessage.__init__(self,
-                                     name=m18ncE('kajongg', 'Mah Jongg'),
-                                     shortcut=m18ncE('kajongg game dialog:Key for Mah Jongg', 'M'))
+                                     name=i18ncE('kajongg', 'Mah Jongg'),
+                                     shortcut=i18ncE('kajongg game dialog:Key for Mah Jongg', 'M'))
 
     def serverAction(self, table, msg):
         """the server mirrors that and tells all others"""
@@ -335,7 +344,7 @@ class MessageMahJongg(NotifyAtOnceMessage, ServerMessage):
 
     def toolTip(self, dummyButton, dummyTile):
         """returns text and warning flag for button and text for tile"""
-        return m18n('Press here and you win'), False, ''
+        return i18n('Press here and you win'), False, ''
 
     def clientAction(self, dummyClient, move):
         """mirror the mahjongg action locally. Check if the balances are correct."""
@@ -349,25 +358,25 @@ class MessageOriginalCall(NotifyAtOnceMessage, ServerMessage):
 
     def __init__(self):
         NotifyAtOnceMessage.__init__(self,
-                                     name=m18ncE('kajongg', 'Original Call'),
-                                     shortcut=m18ncE('kajongg game dialog:Key for Original Call', 'O'))
+                                     name=i18ncE('kajongg', 'Original Call'),
+                                     shortcut=i18ncE('kajongg game dialog:Key for Original Call', 'O'))
 
     def serverAction(self, table, msg):
         """the server tells all others"""
         table.clientDiscarded(msg)
 
     def toolTip(self, button, tile):
-        """decorate the action button which will send this message"""
+        """for the action button which will send this message"""
         assert isinstance(tile, Tile), tile
         myself = button.client.game.myself
         isCalling = bool((myself.hand - tile).callingHands)
         if not isCalling:
-            txt = m18n(
+            txt = i18n(
                 'discarding %1 and declaring Original Call makes this hand unwinnable',
                 tile.name())
             return txt, True, txt
         else:
-            return (m18n(
+            return (i18n(
                 'Discard a tile, declaring Original Call meaning you need only one '
                 'tile to complete the hand and will not alter the hand in any way (except bonus tiles)'),
                     False, '')
@@ -379,7 +388,7 @@ class MessageOriginalCall(NotifyAtOnceMessage, ServerMessage):
             player.originalCallingHand = player.hand
             if Debug.originalCall:
                 logDebug(
-                    u'%s gets originalCallingHand:%s' %
+                    '%s gets originalCallingHand:%s' %
                     (player, player.originalCallingHand))
         player.originalCall = True
         client.game.addCsvTag('originalCall')
@@ -393,15 +402,15 @@ class MessageDiscard(ClientMessage, ServerMessage):
 
     def __init__(self):
         ClientMessage.__init__(self,
-                               name=m18ncE('kajongg', 'Discard'),
-                               shortcut=m18ncE('kajongg game dialog:Key for Discard', 'D'))
+                               name=i18ncE('kajongg', 'Discard'),
+                               shortcut=i18ncE('kajongg game dialog:Key for Discard', 'D'))
 
     def serverAction(self, table, msg):
         """the server mirrors that action"""
         table.clientDiscarded(msg)
 
     def toolTip(self, button, tile):
-        """decorate the action button which will send this message"""
+        """for the action button which will send this message"""
         assert isinstance(tile, Tile), tile
         game = button.client.game
         myself = game.myself
@@ -409,13 +418,13 @@ class MessageDiscard(ClientMessage, ServerMessage):
         warn = False
         if myself.violatesOriginalCall(tile):
             txt.append(
-                m18n('discarding %1 violates Original Call', tile.name()))
+                i18n('discarding %1 violates Original Call', tile.name()))
             warn = True
         if game.dangerousFor(myself, tile):
-            txt.append(m18n('discarding %1 is Dangerous Game', tile.name()))
+            txt.append(i18n('discarding %1 is Dangerous Game', tile.name()))
             warn = True
         if not txt:
-            txt = [m18n('discard the least useful tile')]
+            txt = [i18n('discard the least useful tile')]
         txtStr = '<br><br>'.join(txt)
         return txtStr, warn, txtStr
 
@@ -468,7 +477,7 @@ class MessageReadyForGameStart(ServerMessage):
             if result == Message.OK and client.tableList and client.tableList.isVisible():
                 if Debug.table:
                     logDebug(
-                        u'%s hiding table list because game started' %
+                        '%s hiding table list because game started' %
                         client.name)
                 client.tableList.hide()
             return result
@@ -487,7 +496,7 @@ class MessageNoGameStart(NotifyAtOnceMessage):
 
     def notifyAction(self, client, move):
         if client.beginQuestion or client.game:
-            Sorry(m18n('%1 is not ready to start the game', move.player.name))
+            Sorry(i18n('%1 is not ready to start the game', move.player.name))
         if client.beginQuestion:
             client.beginQuestion.cancel()
         elif client.game:
@@ -521,7 +530,7 @@ class MessageInitHand(ServerMessage):
         scene = Internal.scene
         if scene:
             scene.mainWindow.setWindowTitle(
-                m18n(
+                i18n(
                     'Kajongg <numid>%1</numid>',
                     client.game.handId.seed))
             scene.discardBoard.setRandomPlaces(client.game.randomGenerator)
@@ -599,7 +608,7 @@ class MessageViolatesOriginalCall(ServerMessage):
     def __init__(self):
         ServerMessage.__init__(
             self,
-            name=m18ncE('kajongg',
+            name=i18ncE('kajongg',
                         'Violates Original Call'))
 
     def clientAction(self, client, move):
@@ -607,7 +616,7 @@ class MessageViolatesOriginalCall(ServerMessage):
         move.player.popupMsg(self)
         move.player.mayWin = False
         if Debug.originalCall:
-            logDebug(u'%s: cleared mayWin' % move.player)
+            logDebug('%s: cleared mayWin' % move.player)
         return client.ask(move, [Message.OK])
 
 
@@ -632,7 +641,7 @@ class MessageVoiceData(ServerMessage):
         """server sent us voice sounds about somebody else"""
         move.player.voice = Voice(move.md5sum, move.source)
         if Debug.sound:
-            logDebug(u'%s gets voice data %s from server, language=%s' % (
+            logDebug('%s gets voice data %s from server, language=%s' % (
                 move.player, move.player.voice, move.player.voice.language()))
 
 
@@ -658,7 +667,7 @@ class MessageServerWantsVoiceData(ServerMessage):
     def clientAction(self, dummyClient, move):
         """send voice sounds as requested to server"""
         if Debug.sound:
-            logDebug(u'%s: send wanted voice data %s to server' % (
+            logDebug('%s: send wanted voice data %s to server' % (
                 move.player, move.player.voice))
         return Message.ServerGetsVoiceData, move.player.voice.archiveContent
 
@@ -673,10 +682,10 @@ class MessageServerGetsVoiceData(ClientMessage):
         voice.archiveContent = msg.args[0]
         if Debug.sound:
             if voice.oggFiles():
-                logDebug(u'%s: server got wanted voice data %s' % (
+                logDebug('%s: server got wanted voice data %s' % (
                     msg.player, voice))
             else:
-                logDebug(u'%s: server got empty voice data %s (arg0=%s)' % (
+                logDebug('%s: server got empty voice data %s (arg0=%s)' % (
                     msg.player, voice, repr(msg.args[0][:100])))
 
 
@@ -729,7 +738,7 @@ class MessageDangerousGame(ServerMessage):
     """the game server tells us who played dangerous game"""
 
     def __init__(self):
-        ServerMessage.__init__(self, name=m18ncE('kajongg', 'Dangerous Game'))
+        ServerMessage.__init__(self, name=i18ncE('kajongg', 'Dangerous Game'))
 
     def clientAction(self, client, move):
         """mirror the dangerous game action locally"""
@@ -743,7 +752,7 @@ class MessageNoChoice(ServerMessage):
     """the game server tells us who had no choice avoiding dangerous game"""
 
     def __init__(self):
-        ServerMessage.__init__(self, name=m18ncE('kajongg', 'No Choice'))
+        ServerMessage.__init__(self, name=i18ncE('kajongg', 'No Choice'))
         self.move = None
 
     def clientAction(self, client, move):
@@ -771,7 +780,7 @@ class MessageUsedDangerousFrom(ServerMessage):
         fromPlayer = client.game.playerByName(move.source)
         move.player.usedDangerousFrom = fromPlayer
         if Debug.dangerousGame:
-            logDebug(u'%s claimed a dangerous tile discarded by %s' %
+            logDebug('%s claimed a dangerous tile discarded by %s' %
                      (move.player, fromPlayer))
 
 
@@ -803,12 +812,12 @@ class MessageOK(ClientMessage):
 
     def __init__(self):
         ClientMessage.__init__(self,
-                               name=m18ncE('kajongg', 'OK'),
-                               shortcut=m18ncE('kajongg game dialog:Key for OK', 'O'))
+                               name=i18ncE('kajongg', 'OK'),
+                               shortcut=i18ncE('kajongg game dialog:Key for OK', 'O'))
 
     def toolTip(self, dummyButton, dummyTile):
         """returns text and warning flag for button and text for tile for button and text for tile"""
-        return m18n('Confirm that you saw the message'), False, ''
+        return i18n('Confirm that you saw the message'), False, ''
 
 
 class MessageNoClaim(NotifyAtOnceMessage, ServerMessage):
@@ -817,12 +826,12 @@ class MessageNoClaim(NotifyAtOnceMessage, ServerMessage):
 
     def __init__(self):
         NotifyAtOnceMessage.__init__(self,
-                                     name=m18ncE('kajongg', 'No Claim'),
-                                     shortcut=m18ncE('kajongg game dialog:Key for No claim', 'N'))
+                                     name=i18ncE('kajongg', 'No Claim'),
+                                     shortcut=i18ncE('kajongg game dialog:Key for No claim', 'N'))
 
     def toolTip(self, dummyButton, dummyTile):
         """returns text and warning flag for button and text for tile for button and text for tile"""
-        return m18n('You cannot or do not want to claim this tile'), False, ''
+        return i18n('You cannot or do not want to claim this tile'), False, ''
 
     @classmethod
     def receivers(cls, request):
@@ -843,14 +852,14 @@ def __scanSelf():
                         try:
                             msg = glob()
                         except Exception:
-                            logDebug(u'cannot instantiate %s' % glob.__name__)
+                            logDebug('cannot instantiate %s' % glob.__name__)
                             raise
                         type.__setattr__(
                             Message, msg.name.replace(' ', ''), msg)
                         Message.defined[msg.name] = msg
 
 
-class ChatMessage(object):
+class ChatMessage(StrMixin):
 
     """holds relevant info about a chat message"""
 
@@ -878,17 +887,19 @@ class ChatMessage(object):
             self.timestamp)
         return result + (now - utcnow)
 
-    def __unicode__(self):
+    def __str__(self):
         local = self.localtimestamp()
         # pylint: disable=no-member
         # pylint says something about NotImplemented, check with later versions
-        return u'statusMessage=%s %02d:%02d:%02d %s: %s' % (
-            unicode(self.isStatusMessage),
+        _ = i18n(self.message)
+        if self.isStatusMessage:
+            _ = '[{}]'.format(_)
+        return '%02d:%02d:%02d %s: %s' % (
             local.hour,
             local.minute,
             local.second,
             self.fromUser,
-            m18n(self.message))
+            i18n(self.message))
 
     def asList(self):
         """encode me for network transfer"""
